/*
 * Scratchbox Remote Shell
 * Copyright (c) 2003 Timo Savola <timo.savola@movial.fi>
 *
 * sbrsh is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "config.h"
#include "common.h"
#include "client.h"

/**
 * Compares the beginning of BUF to TGT. Comparison ends at first whitespace.
 */
static char *is_target(char *tgt,
		       char *buf)
{
	int a, b;

	while (1) {
		a = *tgt++;
		b = *buf++;

		if (!a || !b || isspace(b)) {
			return (!a && isspace(b)) ? buf : NULL;
		}

		if (a != b) {
			return NULL;
		}
	}
}

/**
 * Parses BUF into CFG.
 */
static int read_params(char *buf,
		       config_t *cfg)
{
	char *user, *host, *port, *pwd;

	buf = skip_spaces(buf);
	if (!buf) {
		error("Invalid parameter string");
		return 0;
	}

	host = strchr(buf, '@');
	if (host) {
		user = buf;
		*host++ = '\0';
	} else {
		user = NULL;
		host = buf;
	}

	port = strchr(host, ':');
	if (port) {
		*port++ = '\0';
		buf = port;
	} else {
		port = NULL;
		buf = host;
	}

	buf = find_space(buf);
	if (!buf) {
		error("Invalid parameter string");
		return 0;
	}
	*buf++ = '\0';

	pwd = skip_spaces(buf);
	if (!pwd) {
		error("Invalid parameter string");
		return 0;
	}

	cfg->user = user ? strdup(user) : NULL;
	cfg->host = strdup(host);
	cfg->port = port ? atoi(port) : 0;
	cfg->pwd = strdup(pwd);

	return 1;
}

static int read_opts(FILE *file,
		     char *buf,
		     int size,
		     config_t *cfg)
{
	long pos;
	int cnt = 0;
	char **p;

	pos = ftell(file);

	while (read_line(file, buf, size) > 0 &&
	       strlen(skip_spaces(buf)) > 0) {
		cnt++;
	}

	fseek(file, pos, SEEK_SET);

	cfg->opts = calloc(cnt + 1, sizeof (char *));
	if (!cfg->opts) {
		error("Out of memory");
		return 0;
	}

	p = cfg->opts;
	while (cnt-- > 0) {
		read_line(file, buf, size);
		buf = skip_spaces(buf);

		/* Let's be careful. */
		if (strlen(buf) == 0) {
			break;
		}

		*p++ = strdup(buf);
	}

	return 1;
}

int read_config(const char *filename,
		const char *target,
		config_t *cfg)
{
	FILE *file;
	char buf[1024], *buf2;
	int len;
	int ok = 0;

	file = fopen(filename, "r");
	if (!file) {
		return 0;
	}

	while (1) {
		len = read_line(file, buf, sizeof (buf));
		if (len < 0) {
			error("Can't find target %s in config file %s",
			      target, filename);
			break;
		}
		if (len == 0 || isspace(buf[0])) {
			continue;
		}

		buf2 = is_target((char *) target, buf);
		if (buf2) {
			ok = read_params(buf2, cfg) &&
			     read_opts(file, buf, sizeof (buf), cfg);
			break;
		} 
	}

	fclose(file);

	return ok;
}

void init_config(config_t *cfg)
{
	memset(cfg, 0, sizeof (config_t));
}

void free_config(config_t *cfg)
{
	if (cfg->user) {
		free(cfg->user);
	}

	if (cfg->host) {
		free(cfg->host);
	}

	if (cfg->pwd) {
		free(cfg->pwd);
	}

	if (cfg->opts) {
		free_vec((void **) cfg->opts);
	}
}

char **get_targets(const char *filename)
{
	FILE *file;
	char buf[1024], *tmp;
	char **p = NULL;
	int len;
	int cnt = 0;
	int i;
	char **targets = NULL;

	file = fopen(filename, "r");
	if (!file) {
		return NULL;
	}

	/* Go through config two times. First count then store target names */
	for (i=0; i<2; i++) {
		/* Only for second time */
		if (i==1) {
			rewind(file);

			targets = calloc(cnt + 1, sizeof (char *));
			if (! targets) {
				error("Out of memory reading target names");
				return NULL;
			}
			p = targets;
		}
		
		while (1) {
			len = read_line(file, buf, sizeof (buf));
			if (len == 0) {
				continue;
			}
			if (len < 0) {
				break;
			}
			
			if (! isspace(buf[0])) {
				tmp = find_space(buf);
				if (tmp == NULL) {
					error("Something wrong with the config file %s", filename);
					break;
				}

				/* Only for second time */
				if (i==1) {
					*tmp = '\0';
					*p++ = strdup(buf);
				} else {
					cnt++;
				}
			}
		}
	}
	
	return targets;
}

