/*
 * Scratchbox Remote Shell
 * Copyright (c) 2003 Timo Savola <timo.savola@movial.fi>
 *
 * sbrsh is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#ifndef DAEMON_H
#define DAEMON_H

#include <stdio.h>
#include <pwd.h>
#include <netdb.h>
#include "protocol.h"
#include "buffer.h"
#include "version.h"

/** Name of the config file in the user's home directory. */
#define CONFIG_NAME        ".sbrshd"

/** Gets printed with the --version option. */
#define DAEMON_VERSION     "Scratchbox Remote Shell daemon " VERSION

/** Backlog parameter for the listen() call. */
#define BACKLOG            16

/** The permissions used when creating directories for mount points. */
#define MKDIR_PERMS        0755

/** Default path to the mount binary. */
#define DEFAULT_MOUNT_CMD  "/bin/mount"

/** Default path to the umount binary. */
#define DEFAULT_UMOUNT_CMD "/bin/umount"

/** Default path to a file used for checking existing mounts. */
#define DEFAULT_MOUNT_TAB  "/etc/mtab"

/** Default option used with MTYPE_BIND. */
#define DEFAULT_BIND_OPT   "--bind"

/** Default for how many seconds until a mount expires? */
#define DEFAULT_MOUNT_EXPIRATION    (60 * 60)

/** How many times per expiration interval should the mounts be checked? */
#define MOUNT_EXPIRATION_FREQUENCY  6

/** SBRSH_RLIMIT-variable string value that maps to RLIM_INFINITY. */
#define ENV_RLIMIT_UNLIMITED  "unlimited"

/** Prefix for renaming an environment variable. */
#define ENV_RENAME_PREFIX  "SBRSH_ENV_"

/** Value for removing a environment variable. */
#define ENV_RENAME_UNSET   "(UNSET)"

/** Values for the type field of mount_t. */
typedef enum {
	/** mount_t type: NFS. */
	MTYPE_NFS  = 1,

	/** mount_t type: path should be bound to a mount point. */
	MTYPE_BIND = 2
} mtype_t;

struct mount_s;

/** An entry in a doubly linked list of mount points. */
typedef struct mount_s {
	struct mount_s *prev;
	struct mount_s *next;

	/** How many processes are using this mount? */
	int usage;

	/** When will the mount expire? Relevant when usage is 0. */
	time_t expiration;

	/** Mount type. */
	mtype_t type;

	/** Mount point path. */
	char *point;

	/** Path to special file or whatever should be mounted... */
	char *device;

	/** Options for the mount command (-o). */
	char *opts;
} mount_t;

/** Handler process data. */
typedef struct {
	/** Socket connection to the client. */
	int sd;

	/** The protocol version of the client. */
	int client_version;

	/** Host name of the client. */
	char host[NI_MAXHOST];

	/** Username and password from the client. */
	auth_info_t auth;

	/** User's uid, home dir, etc. from the system.
            tmp_buf is used to store the strings of this structure! */
	struct passwd passwd;

	/** Command description from the client. */
	cmd_info_t cmd;

	/** Root directory. */
	char *root;

	/** The mount points used by this client. */
	mount_t **mounts;

	/** Used for buffering stdin data received from the client. */
	buffer_t buf_in;

	/** Used for many things... */
	char tmp_buf[BUFFER_SIZE];

	/** Are we waiting for IN DATA? */
	int inwait;

	/** How much OUT DATA has the client requested? */
	int outreq;

	/** How much ERR DATA has the client requested? */
	int errreq;
} handler_t;

#endif
